@@ -54,6 +54,9 @@
  */
 
 int IsLeapYear (int);
+int match_line_in_db (LogLine *line, GList *db);
+
+extern GList *regexp_db;
 
 /*
  * -------------------
@@ -151,6 +154,7 @@ OpenLogFile (char *filename)
    /* Move end of file marker to last char on file */
    fseek(tlog->fp, -1L, SEEK_END);
    tlog->offset_end = ftell(tlog->fp);
+   tlog->ln = tlog->lstats.numlines - LINES_P_PAGE;
 
    return tlog;
 }
@@ -243,7 +247,7 @@ ReadNPagesUp (Log * lg, Page * pg, int n)
       {
 	 lg->firstpg = cp;
 	 lg->lastpg = cp->prev;
-	 return i;
+	 return i+1;
       }
       cp = cp->prev;
    }
@@ -277,7 +281,7 @@ ReadNPagesDown (Log * lg, Page * pg, int n)
       {
 	 lg->lastpg = cp;
 	 lg->firstpg = cp->prev;
-	 return i;
+	 return i+1;
       }
       cp = cp->next;
    }
@@ -310,6 +314,12 @@ ReadPageUp (Log * lg, Page * pg)
    pg->isfirstpage = FALSE;
    pg->fl = 0;
    pg->ll = LINES_P_PAGE - 1;
+
+   /* Tell if we are reading the last page */
+   if (fgetc (fp) == EOF)
+     pg->islastpage = TRUE;
+   ungetc (ch,fp);
+
    while (ln >= pg->fl)
    {
       c = buffer;
@@ -373,6 +383,11 @@ ReadPageDown (Log * lg, Page * pg)
    pg->isfirstpage = FALSE;
    pg->fl = 0;
    pg->ll = 0;
+
+   /* Check if its the first page */
+   if ( ftell(fp) == 0)
+     pg->isfirstpage = TRUE;
+
    while (ln < LINES_P_PAGE)
    {
       c = buffer;
@@ -415,7 +430,7 @@ ReadPageDown (Log * lg, Page * pg)
       ln++;
    }
    if (pg->ll != LINES_P_PAGE)
-     pg->ll = ln;
+     pg->ll = ln-1;
    else
      pg->ll = LINES_P_PAGE-1;
    pg->lastchpos = ftell (fp);
@@ -434,10 +449,11 @@ void
 ParseLine (char *buff, LogLine * line)
 {
    char *token;
-   char scratch[255];
+   char scratch[1024];
    int i;
 
    token = strtok (buff, " ");
+   if (token == NULL) return;
    if (strlen (token) != 3)
    {
       strncpy (line->message, buff, MAX_WIDTH);
@@ -538,6 +554,17 @@ ParseLine (char *buff, LogLine * line)
 	 token++;
       strncpy (line->message, token, MAX_WIDTH);
    }
+
+   /* Search current data base for string and attach descritpion */
+   /* -------------------------
+   if (regexp_db == NULL)
+     return;
+   if (match_line_in_db (line, regexp_db))
+     {
+       sprintf (scratch, "Expression /%s/ matched.", line->description->regexp);
+       line->description->description = g_strdup(scratch);
+     }
+     ---------------------------- */
 }
 
 /* ----------------------------------------------------------------------
@@ -605,6 +632,7 @@ ReadLogStats (Log * log)
    curmark->next = NULL;
    curmark->prev = NULL;
    curmark->offset = filemark;
+   curmark->ln = nl;
 
    /* Count lines up to end */
    filemark = ftell (fp);
@@ -637,6 +665,7 @@ ReadLogStats (Log * log)
      curmark->year = offsetyear;
      curmark->next = NULL;
      curmark->offset = filemark;
+     curmark->ln = nl;
      curdate = newdate;
      filemark = ftell (fp);
    }
@@ -776,56 +805,59 @@ UpdateLogStats( Log *log )
 void
 MoveToMark (Log *log)
 {
-   DateMark *mark;
-   FILE *fp;
-   Page *middlepg;
-   int pagesdown, i;
-
-   if (log == NULL)
-     return;
-
-   mark = log->curmark;
-   fp = log->fp;
-
-   pagesdown = NUM_PAGES>>1;
-
-   /* Read relative to middle page */
-   middlepg = log->lastpg;
-   for(i=0;i<pagesdown;i++)
-     middlepg = middlepg->prev;
-
-   /* Move file pointer to the current offset 
-      into the file */
-	if (mark->offset > 1)
-	  fseek(fp, mark->offset-1, SEEK_SET);
-	else
-	  fseek(fp, mark->offset, SEEK_SET);
-   ReadPageDown(log, middlepg);
-   if (middlepg->islastpage)
-     {
-     middlepg = log->lastpg;
-     fseek(fp, mark->offset-1, SEEK_SET);
-     ReadPageDown(log, middlepg);
-     ReadNPagesUp(log, middlepg, NUM_PAGES-1);
-     log->currentpg = middlepg;
-     return;
-     }
-
-   if (middlepg->isfirstpage)
-     {
-     middlepg = log->firstpg;
-     fseek(fp, mark->offset, SEEK_SET);
-     ReadPageDown(log, middlepg);
-     ReadNPagesDown(log, middlepg, NUM_PAGES-1);
-     log->currentpg = middlepg;
-     return;
-     }
-
-   ReadNPagesDown(log, middlepg, pagesdown);
-   ReadNPagesUp(log, middlepg, NUM_PAGES-1-pagesdown);
-
-   log->currentpg = middlepg;
+  DateMark *mark;
+  FILE *fp;
+  Page *middlepg;
+  int pagesread, pagesdown, i;
+  
+  if (log == NULL)
+    return;
+  
+  mark = log->curmark;
+  fp = log->fp;
+  
+  pagesdown = NUM_PAGES>>1;
+  
+  /* Read relative to middle page */
+  middlepg = log->lastpg;
+  for(i=0;i<pagesdown;i++)
+    middlepg = middlepg->prev;
+  
+  /* Move file pointer to the current offset 
+     into the file */
+  if (mark->offset > 1)
+    fseek(fp, mark->offset-1, SEEK_SET);
+  else
+    fseek(fp, mark->offset, SEEK_SET);
+  ReadPageDown(log, middlepg);
+  if (middlepg->islastpage)
+    {
+      middlepg = log->lastpg;
+      if (mark->offset > 1)
+	fseek(fp, mark->offset-1, SEEK_SET);
+      else
+	fseek(fp, mark->offset, SEEK_SET);
+      ReadPageDown(log, middlepg);
+      ReadNPagesUp(log, middlepg, NUM_PAGES-1);
+      log->currentpg = middlepg;
+      return;
+    }
+  
+  if (middlepg->isfirstpage)
+    {
+      middlepg = log->firstpg;
+      fseek(fp, mark->offset, SEEK_SET);
+      ReadPageDown(log, middlepg);
+      ReadNPagesDown(log, middlepg, NUM_PAGES-1);
+      log->currentpg = middlepg;
+      return;
+    }
+  
+  pagesread = ReadNPagesDown (log, middlepg, NUM_PAGES-1);
+  ReadNPagesUp (log, middlepg, MAX (pagesdown, (NUM_PAGES-1-pagesread)));
 
+  log->currentpg = middlepg;
+   
 }
 
 
